from euclid import *
import texman
from pyglet.gl import *

class Actor(object):
    def __init__(self,pos, **kwargs):
        self.__dict__.update(kwargs)
        self.position = Vector3(*pos)
        self.ticks = 0
        self.groups = []
    
    def add(self, group):
        group.add(self)
        self.groups.append(group)

    def die(self):
        for g in self.groups:
            g.remove(self)

    def draw(self):
        pass
    def _think(self, dt, gamestate=None, controls=None):
        pass

    def think(self, dt, gamestate=None, controls=None):
        self.ticks += 1
        self._think(dt, gamestate,controls)

class BoardActor(Actor):

    def __init__(self, pos, **kwargs):
        
        self.scale = (1,1,1)
        self.rotation = 0
        self.rotations = [0] * 10
        Actor.__init__(self, pos, **kwargs)
        self.img = texman.get_tex(self.imgname).frames[0]
        gwidth = self.img.width / 2.0 
        gheight = self.img.height
        self.verts = (GLfloat * 8)(-gwidth,gheight*.5,gwidth,gheight*.5,gwidth,-gheight*.5,-gwidth,-gheight*.5)
        self.texcoords = (GLfloat * 8)(0,1,1,1,1,0,0,0)
    def draw(self):
        x, y, z = self.position
        
        glTranslatef(*self.position)
        glRotatef(self.rotation,0,0,1)
        glScalef(*self.scale)
        glEnable(GL_TEXTURE_2D)
        glBindTexture(GL_TEXTURE_2D, self.img.id)
        glColor4f(1,1,1,1)
        glEnableClientState(GL_VERTEX_ARRAY)
        glEnableClientState(GL_TEXTURE_COORD_ARRAY)
        glVertexPointer(2, GL_FLOAT, 0, self.verts)
        glTexCoordPointer(2, GL_FLOAT, 0, self.texcoords)
        glDrawArrays(GL_QUADS, 0, len(self.verts) // 2)
        
class Ship(BoardActor):
    def __init__(self, pos, **kwargs):
        BoardActor.__init__(self, pos,imgname='ship',**kwargs)
    
    def _think(self, dt, gamestate=None, controls=None):
        self.position.x += controls.dx
        self.position.y -= controls.dy
        
        self.rotations = self.rotations[1:]
        self.rotations.append(controls.dx * -1)
        self.rotation = sum(self.rotations) / 10.0
        controls.dx = 0
        controls.dy = 0