from pyglet.gl import *
import engine
import texman
import conf
import terrain
import random
import actor
from euclid import *

class FlightEngine(engine.Engine):
    def __init__(self):
        engine.Engine.__init__(self)
        self.fogcolor = (GLfloat *4)(.8,.8,.9,1)
        self.skycolor = (GLfloat *4)(.2,.2,.7,1)
        glClearColor(*self.fogcolor)
        self.scrollspeed = 1.0
        self.terrain = set()
        for z in range(conf.near_plane, conf.far_plane, conf.terrain_spacing):
            g = terrain.Grass((random.random()*60,0, z))
            g.add(self.terrain)
        self.rotation = 0
        self.velocity = 2.3;
        self.scroll = 0
        self.last_add = 0
        self.ship = actor.Ship((0, 0, 0))
        self.camerapos = Vector3(0,200,0)
        
    def _set_projection(self):
        glMatrixMode(GL_PROJECTION)
        glLoadIdentity()
        self.view_width = conf.viewport_size[0] * .5
        self.view_height = conf.viewport_size[1] * .5
        depth = conf.far_plane - conf.near_plane
        near_mult = 1 + conf.near_plane / (float(-conf.near_plane + conf.focal_length))
        glFrustum(-self.view_width * near_mult,
                  self.view_width * near_mult,
                  -self.view_height * near_mult,
                  self.view_height * near_mult,
                  conf.focal_length ,
                  conf.focal_length + depth)
        glTranslatef(0,0, -conf.focal_length + conf.near_plane )
        glScalef(1,1,-1)
        #gluOrtho2D(0,640,0,480)
        glMatrixMode(GL_MODELVIEW)
        glLoadIdentity()
        
    def draw_sky(self, rotation):
        glDisable(GL_FOG)
        glRotatef(rotation,0,0,1)
        glBegin(GL_QUADS)
        glColor4fv(self.skycolor)
        glVertex2f(-1000,self.view_height*2)
        glVertex2f(1000,self.view_height*2)
        glColor4fv(self.fogcolor)
        glVertex2f(1000,0)
        glVertex2f(-1000,0)
        
        glEnd()
        glRotatef(-rotation,0,0,1)
        glClear(GL_DEPTH_BUFFER_BIT)
    def follow_player(self):
        pdiff = self.ship.position - self.camerapos
        self.camerapos += pdiff * .1
        
    def setCamera(self):
        glRotatef(self.rotation,0,0,1)
        glTranslatef(*(-self.camerapos))
        
    @classmethod
    def depth_sort(cls, lst):
        sorted = list(lst)
        sorted.sort(key=lambda x:x.position.z, reverse=True)
        return sorted
    
    def _update(self, window, user_input, dt):
        self.scroll_actors(self.terrain)
        self.scroll += self.velocity
        
        for t in self.terrain:
            t.think(dt)
        self.ship.think(dt, controls=user_input)
        self.follow_player()
        self.add_terrain()
        
    def add_terrain(self):
        if self.last_add > conf.terrain_spacing:
            self.last_add -= conf.terrain_spacing
            g = terrain.Grass((random.random()*60,0, conf.far_plane - self.last_add))
            g.add(self.terrain)
        if random.random()<.04:
            r = terrain.Rock((-1200 + random.random() * 2400, 0, conf.far_plane - self.last_add + .1))
            r.add(self.terrain)
        self.last_add += self.velocity
        
    def scroll_actors(self, actorlist):
        for a in list(actorlist):
            a.position.z -= self.velocity
            if a.position.z < conf.near_plane:
                a.die()
        
    def _render(self, window):
        glEnable(GL_BLEND)
        self._set_projection()
        self.rotation = self.ship.rotation * .2
        self.draw_sky(self.rotation)
        self.setCamera()
        
        glEnable(GL_FOG)
        glFogi(GL_FOG_MODE, GL_LINEAR)
        glFogfv(GL_FOG_COLOR, self.fogcolor)
        glFogf(GL_FOG_DENSITY, 2)
        glHint(GL_FOG_HINT, GL_NICEST)
        glFogf(GL_FOG_START, conf.far_plane * .2 )
        glFogf(GL_FOG_END, conf.far_plane)
        glMatrixMode(GL_MODELVIEW)
        
        to_draw = self.depth_sort(list(self.terrain) + [self.ship])
        for t in to_draw:
            glPushMatrix()
            t.draw()
            glPopMatrix()

